@@ -3,21 +3,20 @@ package Win32::Unicode::Process;
 use strict;
 use warnings;
 use 5.008003;
-use Win32::API ();
 use Win32API::File ();
 use Carp ();
 use Exporter 'import';
 
 use Win32::Unicode::Util;
 use Win32::Unicode::Constant;
-use Win32::Unicode::Define;
+use Win32::Unicode::XS;
 
 # export subs
 our @EXPORT    = qw/systemW execW/;
 our @EXPORT_OK = qw//;
 our %EXPORT_TAGS = ('all' => [@EXPORT, @EXPORT_OK]);
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 # cmd path
 my $SHELL = do {
@@ -27,18 +26,19 @@ my $SHELL = do {
 
 sub systemW {
     my $pi = _create_process(@_) or return 1;
-    Win32API::File::CloseHandle($pi->{hThread});
-    WaitForInputIdle->Call($pi->{hProcess}, INFINITE);
-    WaitForSingleObject->Call($pi->{hProcess}, INFINITE);
-    Win32API::File::CloseHandle($pi->{hProcess});
+    Win32API::File::CloseHandle($pi->{thread_handle});
+    wait_for_input_idle($pi->{process_handle});
+    wait_for_single_object($pi->{process_handle});
+    my $exit_code = get_exit_code($pi->{process_handle});
+    Win32API::File::CloseHandle($pi->{process_handle});
     
-    return 0;
+    return defined $exit_code ? $exit_code : 1;
 }
 
 sub execW {
     my $pi = _create_process(@_) or return 1;
-    Win32API::File::CloseHandle($pi->{hThread});
-    Win32API::File::CloseHandle($pi->{hProcess});
+    Win32API::File::CloseHandle($pi->{thread_handle});
+    Win32API::File::CloseHandle($pi->{process_handle});
     
     return 0;
 }
@@ -55,23 +55,7 @@ sub _create_process {
     
     $cmd = utf8_to_utf16("/x /c $cmd " . join q{ }, @args) . NULL; # mybe security hole :-(
     
-    my $si = Win32::API::Struct->new('STARTUPINFO');
-    my $pi = Win32::API::Struct->new('PROCESS_INFORMATION');
-    
-    CreateProcess->Call(
-        $SHELL,
-        $cmd,
-        0,
-        0,
-        FALSE,
-        NORMAL_PRIORITY_CLASS,
-        0,
-        0,
-        $si,
-        $pi,
-    ) or return;
-    
-    return $pi;
+    return create_process($SHELL, $cmd);
 }
 
 1;
